/*
	Copyright (C) 2011 Micha Hersch, University of Lausanne, Swiss Institute of Bioinformatics

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __OPEN_COTYLEDON_H__
#define __OPEN_COTYLEDON_H__

#include  "ip_public.h" 
#include "basicMath.h"


//#define OPEN_COTYLEDON_DEV //used for development purposes

#ifdef OPEN_COTYLEDON_DEV
#include "cotyledon.h"
#endif

/**
 * A class for tracking the open cotyledon of a seedling on an image. 
 * 
 */
class OpenCotyledon{

 protected:
  Vec2 position;
  Vec2 old_position;

  Vec2 win_size;
  Vec2 *pre_features;
  Vec2 *post_features;
  char *of_feature_found;
  f32 *of_feature_error;

  int maxFeatures;

  int active;

#ifdef OPEN_COTYLEDON_DEV
  CotyledonTuner tuner;
#endif

 public:
  OpenCotyledon();
  int computeOF(IplImage *img,IplImage *img_old );
  Vec2 update(IplImage *img,IplImage *img_old);
  void displayFeatures(IplImage *img,int n,const char *wname);
  void displayMatchingFeatures(IplImage *img,int n,const char *wname);
  void setPosition(Vec2 p){position = p;}
  Vec2 getPosition()const {return position;}
  void hide(IplImage *img);
  int isDrawable(){return active;}
  void inactivate(){active=0;}
  void draw(IplImage *img,int ox=0,int oy=0,CvScalar col= CV_RGB(0,255,0),
	    int thick=1)
  {cvCircle(img,point2(position-Vec2(ox,oy)),2,col,1);}
  void translate(Vec2 trans){position += trans;}
#ifdef OPEN_COTYLEDON_DEV
  void runTuner(const char *imfname);
  void displayPosition(const char *wname){displayMatchingFeatures(0,wname);}
#endif
};

static std::ostream& operator<<(std::ostream& out, const OpenCotyledon& cot){
  out<<cot.getPosition();
  return out;
}

static std::istream& operator>>(std::istream& in, OpenCotyledon& cot){
  Vec2 v1;
  in>>v1;
  cot.setPosition(v1);
  return in;
}

#endif
